//
// Written by Sander van Rossen
//	You are free to use this piece of code any way you like
//
#pragma once



/*! 
	@brief		Used to describe the order of the color components in a color
*/
enum EColorType
{
	COLOR_NONE				,	///< Color has no components
	
	COLOR_RGBA				,	///< Color is ordered as RGBA
	COLOR_BGRA				,	///< Color is ordered as BGRA
	COLOR_ARGB				,	///< Color is ordered as ARGB
	COLOR_ABGR				,	///< Color is ordered as ABGR
	COLOR_RGB				,	///< Color is ordered as RGB (no ALPHA channel)
	COLOR_BGR				,	///< Color is ordered as BGR (no ALPHA channel)
	COLOR_RED				,	///< Color only has a RED color channel
	
	COLOR_GREEN				,	///< Color only has a GREEN color channel
	COLOR_BLUE				,	///< Color only has a BLUE color channel
	COLOR_ALPHA					///< Color only has an ALPHA channel

};//EColorType	




/*! 
	@brief		ColorDescription describes a color and it's components
*/
template
<
	EColorType	tColorOrder	,	///< the order of all the color components

	cUInt		tRedSize	,	///< number of RED color component bits
	cUInt		tGreenSize	,	///< number of GREEN color component bits
	cUInt		tBlueSize	,	///< number of BLUE color component bits
	cUInt		tAlphaSize	,	///< number of ALPHA component bits
	cUInt		tSpacing		///< number of bits that are left unused
>
class ColorDescription
{
public:
	enum EColorConstants
	{
		//
		// Sizes of individual color components
		//
		RED_SIZE			= tRedSize		,	///< Size of the RED color component
		GREEN_SIZE			= tGreenSize	,	///< Size of the GREEN color component
		BLUE_SIZE			= tBlueSize		,	///< Size of the BLUE color component
		ALPHA_SIZE			= tAlphaSize	,	///< Size of the ALPHA component
		SPACING_SIZE		= tSpacing		,	///< Size of the unused space



		///
		/// Actuall size of color
		///
		COLOR_BIT_SIZE		= RED_SIZE		+ 
							  GREEN_SIZE	+ 
							  BLUE_SIZE		+ 
							  ALPHA_SIZE	,


		
		///
		/// Storage size of color
		///
		COLOR_STORAGE_BITS	= RED_SIZE		+ 
							  GREEN_SIZE	+ 
							  BLUE_SIZE		+ 
							  ALPHA_SIZE	+ 
							  SPACING_SIZE	,



		///
		/// REMOVE_COMPONENT is used to shift a color component to outside an int
		/// this way the color component is simply removed instead of used in the
		/// calculation
		///
		REMOVE_COMPONENT	= BYTES_PER_INT ,

		

		//
		// Helpers to shorten the notation when calculating the position of each
		// component
		//
		POS_0				= 0,												///< at the start of the color
		POS_R				= RED_SIZE										,	///< after the RED component
		POS_G				=			 GREEN_SIZE							,	///< after the GREEN component
		POS_RG				= RED_SIZE + GREEN_SIZE							,	///< after the RED and GREEN components
		POS_B				=						  BLUE_SIZE				,	///< after the BLUE component
		POS_RB				= RED_SIZE +			  BLUE_SIZE				,	///< after the RED and BLUE components
		POS_GB				=			 GREEN_SIZE + BLUE_SIZE				,	///< after the GREEN and BLUE components
		POS_RGB				= RED_SIZE + GREEN_SIZE + BLUE_SIZE				,	///< after the RED, GREEN and BLUE components
		POS_A				=									  ALPHA_SIZE,	///< after the ALPHA component
		POS_RA				= RED_SIZE +						  ALPHA_SIZE,	///< after the RED and ALPHA components
		POS_GA				=			 GREEN_SIZE +			  ALPHA_SIZE,	///< after the GREEN and ALPHA components
		POS_RGA				= RED_SIZE + GREEN_SIZE +			  ALPHA_SIZE,	///< after the RED, GREEN and ALPHA components
		POS_BA				=						  BLUE_SIZE + ALPHA_SIZE,	///< after the RED, BLUE and ALPHA components
		POS_RBA				= RED_SIZE + 			  BLUE_SIZE + ALPHA_SIZE,	///< after the BLUE and ALPHA components
		POS_GBA				=			 GREEN_SIZE + BLUE_SIZE + ALPHA_SIZE,	///< after the GREEN, BLUE and ALPHA components
		POS_RGBA			= RED_SIZE + GREEN_SIZE + BLUE_SIZE + ALPHA_SIZE,	///< after the RED, GREEN, BLUE and ALPHA components
		


		//
		// Calculate the position of each color component in the color
		//

		/// Calculates the offset of the RED color component in the color
		///		This 'switch' statement is calculated at compile time
		RED_OFFSET			= (tColorOrder == COLOR_NONE )? REMOVE_COMPONENT:	///< this type of color has no components
							  (tColorOrder == COLOR_RGBA )? POS_0			:	///< component starts at the beginning of the color
							  (tColorOrder == COLOR_BGRA )? POS_GB			:	///< component starts after the GREEN and BLUE components
							  (tColorOrder == COLOR_ARGB )? POS_A			:	///< component starts after the ALPHA component
							  (tColorOrder == COLOR_ABGR )? POS_GBA			:	///< component starts after the GREEN, BLUE and ALPHA components
							  (tColorOrder == COLOR_RGB  )? POS_0			:	///< component starts at the beginning of the color
							  (tColorOrder == COLOR_BGR  )? POS_GB			:	///< component starts after the GREEN and BLUE components
							  (tColorOrder == COLOR_RED	 )? POS_0			:	///< component starts at the beginning of the color
							  (tColorOrder == COLOR_GREEN)? REMOVE_COMPONENT:	///< this type of color has no RED
							  (tColorOrder == COLOR_BLUE )? REMOVE_COMPONENT:	///< this type of color has no RED
							  (tColorOrder == COLOR_ALPHA)? REMOVE_COMPONENT:	///< this type of color has no RED
							  REMOVE_COMPONENT,

		/// Calculates the offset of the GREEN color component in the color
		///		This 'switch' statement is calculated at compile time
		GREEN_OFFSET		= (tColorOrder == COLOR_NONE )? REMOVE_COMPONENT:	///< this type of color has no components
							  (tColorOrder == COLOR_RGBA )? POS_R			:	///< component starts after the RED component
							  (tColorOrder == COLOR_BGRA )? POS_B			:	///< component starts after the BLUE component
							  (tColorOrder == COLOR_ARGB )? POS_RA			:	///< component starts after the GREEN and ALPHA components
							  (tColorOrder == COLOR_ABGR )? POS_BA			:	///< component starts after the BLUE and ALPHA components
							  (tColorOrder == COLOR_RGB  )? POS_R			:	///< component starts after the RED component
							  (tColorOrder == COLOR_BGR  )? POS_B			:	///< component starts after the BLUE component
							  (tColorOrder == COLOR_RED	 )? REMOVE_COMPONENT:	///< this type of color has no GREEN
							  (tColorOrder == COLOR_GREEN)? POS_0			:	///< component starts at the beginning of the color
							  (tColorOrder == COLOR_BLUE )? REMOVE_COMPONENT:	///< this type of color has no GREEN
							  (tColorOrder == COLOR_ALPHA)? REMOVE_COMPONENT:	///< this type of color has no GREEN
							  REMOVE_COMPONENT,

		/// Calculates the offset of the BLUE color component in the color
		///		This 'switch' statement is calculated at compile time
		BLUE_OFFSET			= (tColorOrder == COLOR_NONE )? REMOVE_COMPONENT:	///< this type of color has no components
							  (tColorOrder == COLOR_RGBA )? POS_RG			:	///< component starts after the RED and GREEN components
							  (tColorOrder == COLOR_BGRA )? POS_0			:	///< component starts at the beginning of the color
							  (tColorOrder == COLOR_ARGB )? POS_RGA			:	///< component starts after the RED, GREEN and ALPHA components
							  (tColorOrder == COLOR_ABGR )? POS_A			:	///< component starts after the ALPHA component
							  (tColorOrder == COLOR_RGB  )? POS_RG			:	///< component starts after the RED and GREEN components
							  (tColorOrder == COLOR_BGR  )? POS_0			:	///< component starts at the beginning of the color
							  (tColorOrder == COLOR_RED	 )? REMOVE_COMPONENT:	///< this type of color has no BLUE
							  (tColorOrder == COLOR_GREEN)? REMOVE_COMPONENT:	///< this type of color has no BLUE
							  (tColorOrder == COLOR_BLUE )? POS_0			:	///< component starts at the beginning of the color
							  (tColorOrder == COLOR_ALPHA)? REMOVE_COMPONENT:	///< this type of color has no BLUE
							  REMOVE_COMPONENT,

		/// Calculates the offset of the ALPHA component in the color
		///		This 'switch' statement is calculated at compile time
		ALPHA_OFFSET		= (tColorOrder == COLOR_NONE )? REMOVE_COMPONENT:	///< this type of color has no components
							  (tColorOrder == COLOR_RGBA )? POS_RGB			:	///< component starts after the RED, GREEN and BLUE components
							  (tColorOrder == COLOR_BGRA )? POS_RGB			:	///< component starts after the RED, GREEN and BLUE components
							  (tColorOrder == COLOR_ARGB )? POS_0			:	///< component starts at the beginning of the color
							  (tColorOrder == COLOR_ABGR )? POS_0			:	///< component starts at the beginning of the color
							  (tColorOrder == COLOR_RGB  )? REMOVE_COMPONENT:	///< this type of color has no ALPHA
							  (tColorOrder == COLOR_BGR  )? REMOVE_COMPONENT:	///< this type of color has no ALPHA
							  (tColorOrder == COLOR_RED	 )? REMOVE_COMPONENT:	///< this type of color has no ALPHA
							  (tColorOrder == COLOR_GREEN)? REMOVE_COMPONENT:	///< this type of color has no ALPHA
							  (tColorOrder == COLOR_BLUE )? REMOVE_COMPONENT:	///< this type of color has no ALPHA
							  (tColorOrder == COLOR_ALPHA)? POS_0			:	///< component starts at the beginning of the color
							  REMOVE_COMPONENT,


		///
		/// Set if the current color has an ALPHA channel
		///
		HAS_ALPHA			= ALPHA_SIZE	!= 0,

		
		///
		/// Set if the current color IS an ALPHA channel
		///
		IS_ALPHA			= RED_SIZE		== 0 && 
							  GREEN_SIZE	== 0 && 
							  BLUE_SIZE		== 0 && 
							  ALPHA_SIZE	!= 0
	};//EColorConstants


};//ColorDescription
